
#ifndef _CRACKERS_SIDEWAYS_H_
#define _CRACKERS_SIDEWAYS_H_

struct mapNode{
        int             tailBatId;	  
        struct mapNode  *nextMap;
        struct Node     *nextPiece;
};

struct mapSetNode{
        int             batId;
        struct mapNode  *nextMap;
        struct mapSetNode *nextMapSet;
};

struct storageManagement{
	oid totalTuples;	
	oid threshold;
	bit setThreshold;
	struct mapSetNode *nextMapSet;	/* the list of map set nodes are holding info on map sets based on mapSetNode->batId
					   Each map set node has a list of mapNodes for each map that exists in this map set
					   Each mapNode has a list of Nodes linking directly to the cracker indices of the respective maps
					   Each Node is the FIRST node of a piece in a map*/

	struct referencesNodeFM *references; /* used in the case of full maps to hold the maps used in the current query */
	int temp;
};

#ifdef LIBCRACKERS
/* Signatures shared within the crackers module/library */

int existsCrackerIndexSideways(int bid, int pbid);





int CreateTape_int(int bid);
str AddToTape_int(int h, int low, int hgh, bit li, bit hi);
str AddToTapeIncrementalSideways_int(struct Node *node, int value, bit inclusive);




int CreateTape_date(int bid);
str AddToTape_date(int h, date low, date hgh, bit li, bit hi);
str AddToTapeIncrementalSideways_date(struct Node *node, date value, bit inclusive);







/* TODO: add the rest of the type combinations once code is stable */

str Sync_int_int(int m, int h);

str CRKcrackUnorderedZeroSideways_LE_int_int( BAT *b, int mval, oid first, oid last, oid *pos);




str CRKcrackUnorderedZeroSideways_RE_int_int( BAT *b, int mval, oid first, oid last, oid *pos);







str Sync_int_date(int m, int h);

str CRKcrackUnorderedZeroSideways_LE_int_date( BAT *b, int mval, oid first, oid last, oid *pos);




str CRKcrackUnorderedZeroSideways_RE_int_date( BAT *b, int mval, oid first, oid last, oid *pos);








str Sync_date_int(int m, int h);

str CRKcrackUnorderedZeroSideways_LE_date_int( BAT *b, date mval, oid first, oid last, oid *pos);




str CRKcrackUnorderedZeroSideways_RE_date_int( BAT *b, date mval, oid first, oid last, oid *pos);







str Sync_date_lng(int m, int h);

str CRKcrackUnorderedZeroSideways_LE_date_lng( BAT *b, date mval, oid first, oid last, oid *pos);




str CRKcrackUnorderedZeroSideways_RE_date_lng( BAT *b, date mval, oid first, oid last, oid *pos);







str Sync_date_date(int m, int h);

str CRKcrackUnorderedZeroSideways_LE_date_date( BAT *b, date mval, oid first, oid last, oid *pos);




str CRKcrackUnorderedZeroSideways_RE_date_date( BAT *b, date mval, oid first, oid last, oid *pos);









#endif

/* Exported signatures */

crackers_export str CRKsizePendingInsertions(int *k, int *bid, int *pbid);
crackers_export str CRKextendCrackerMap(int *k, int *bid, int *pbid, oid positions);
crackers_export str CRKprintPendingInsertions(int *k, int *bid, int  *pbid);
crackers_export str CRKinsert(int *k, int *bid, int *pbid, int *insertions);
crackers_export str DeleteCrackerMap(int *k, int *bid, int *pbid);
crackers_export str getCrackerMap(int *k, int *bid, int *pbid);
crackers_export str CRKgetTotalStorage(int *k);
crackers_export str CRKsetStorageThreshold(int *k, oid * threshold);
crackers_export str CRKaddReference(int* res, int *bid, int *pbid);
crackers_export str CRKaddReferenceFullMaps(int* res, int *bid, int *pbid);
crackers_export str CRKclearReferences(int* res, int *bid);
crackers_export str CRKclearReferencesFullMaps(int* res);
crackers_export str CRKremoveFullMap(int* res);





/* JoinSelectFunctionsSidewaysWithoutPivot */
crackers_export str CRKjoinSelectSideways_int(int *res, int *cpair, int *low, int *hgh, bit *inclusiveLow, bit *inclusiveHgh);




/* JoinSelectFunctionsSidewaysWithoutPivot */
crackers_export str CRKjoinSelectSideways_lng(int *res, int *cpair, lng *low, lng *hgh, bit *inclusiveLow, bit *inclusiveHgh);




/* JoinSelectFunctionsSidewaysWithoutPivot */
crackers_export str CRKjoinSelectSideways_date(int *res, int *cpair, date *low, date *hgh, bit *inclusiveLow, bit *inclusiveHgh);







/* TODO: add the rest of the type combinations once code is stable */

/* SelectFunctionsSideways */
crackers_export str CRKcreateFullMap_int_int(int* res, int *bid, int *pbid);
crackers_export str CRKhselectBoundsSideways_int_int(int *vid, int *bid, int *pbid, int *low, int *hgh, bit *inclusiveLow, bit *inclusiveHgh);
crackers_export str CRKtselectBoundsSideways_int_int(int *vid, int *bid, int *pbid, int *low, int *hgh, bit *inclusiveLow, bit *inclusiveHgh);
crackers_export str CRKdselectBoundsSideways_int_int(int *vid, int *bid, int *pbid, int *low, int *hgh, bit *inclusiveLow, bit *inclusiveHgh, int *low2, int *hgh2, bit *inclusiveLow2, bit *inclusiveHgh2);
crackers_export str CRKselectBoundsIncrementalSideways_int_int(int *vid, int *bid, int *pbid, int *low, int *hgh, bit *inclusiveLow, bit *inclusiveHgh, int *low2, int *hgh2, bit *inclusiveLow2, bit *inclusiveHgh2);
crackers_export str CRKselectBoundsTailIncrementalSideways_int_int(int *vid, int *bid, int *pbid, int *low, int *hgh, bit *inclusiveLow, bit *inclusiveHgh);
/* JoinSelectFunctionsSideways */
crackers_export str CRKjoinSelectSideways_P_int_int(int *res, int *bid, int *pbid, int *pivot, int *low1, int *hgh1, bit *inclusiveLow1, bit *inclusiveHgh1, int *low, int *hgh, bit *inclusiveLow, bit *inclusiveHgh);
crackers_export str CRKjoinSelectIncrementalSideways_int_int(int *res, int *bid, int *pbid, int *pivot, int *low1, int *hgh1, bit *inclusiveLow1, bit *inclusiveHgh1, int *low, int *hgh, bit *inclusiveLow, bit *inclusiveHgh);
crackers_export str CRKProjectIncrementalSideways_int_int(int *res, int *bid, int *pbid, int *pivot, int *low1, int *hgh1, bit *inclusiveLow1, bit *inclusiveHgh1);
crackers_export str CRKMaxTailIncrementalSideways_int_int(int *res, int *bid, int *pbid, int *low1, int *hgh1, bit *inclusiveLow1, bit *inclusiveHgh1);
crackers_export str CRKdjoinSelectSideways_P_int_int(int *res, int *bid, int *pbid, int *pivot, int *low, int *hgh, bit *inclusiveLow, bit *inclusiveHgh);
/* Project */
crackers_export str CRKProjectCrack_int_int(int *res, int *bid, int *pbid, int *low, int *hgh, bit *inclusiveLow, bit *inclusiveHgh);
crackers_export str CRKProjectH_int_int(int *res, int *bid, int *pbid, int *pivot, int *low, int *hgh, bit *inclusiveLow, bit *inclusiveHgh);
crackers_export str CRKProject_int_int(int *res, int *bid, int *pbid, int *pivot, int *low, int *hgh, bit *inclusiveLow, bit *inclusiveHgh);
crackers_export str CRKdProject_int_int(int *res, int *bid, int *pbid, int *pivot);




/* SelectFunctionsSideways */
crackers_export str CRKcreateFullMap_int_date(int* res, int *bid, int *pbid);
crackers_export str CRKhselectBoundsSideways_int_date(int *vid, int *bid, int *pbid, int *low, int *hgh, bit *inclusiveLow, bit *inclusiveHgh);
crackers_export str CRKtselectBoundsSideways_int_date(int *vid, int *bid, int *pbid, int *low, int *hgh, bit *inclusiveLow, bit *inclusiveHgh);
crackers_export str CRKdselectBoundsSideways_int_date(int *vid, int *bid, int *pbid, int *low, int *hgh, bit *inclusiveLow, bit *inclusiveHgh, date *low2, date *hgh2, bit *inclusiveLow2, bit *inclusiveHgh2);
crackers_export str CRKselectBoundsIncrementalSideways_int_date(int *vid, int *bid, int *pbid, int *low, int *hgh, bit *inclusiveLow, bit *inclusiveHgh, date *low2, date *hgh2, bit *inclusiveLow2, bit *inclusiveHgh2);
crackers_export str CRKselectBoundsTailIncrementalSideways_int_date(int *vid, int *bid, int *pbid, int *low, int *hgh, bit *inclusiveLow, bit *inclusiveHgh);
/* JoinSelectFunctionsSideways */
crackers_export str CRKjoinSelectSideways_P_int_date(int *res, int *bid, int *pbid, int *pivot, int *low1, int *hgh1, bit *inclusiveLow1, bit *inclusiveHgh1, date *low, date *hgh, bit *inclusiveLow, bit *inclusiveHgh);
crackers_export str CRKjoinSelectIncrementalSideways_int_date(int *res, int *bid, int *pbid, int *pivot, int *low1, int *hgh1, bit *inclusiveLow1, bit *inclusiveHgh1, date *low, date *hgh, bit *inclusiveLow, bit *inclusiveHgh);
crackers_export str CRKProjectIncrementalSideways_int_date(int *res, int *bid, int *pbid, int *pivot, int *low1, int *hgh1, bit *inclusiveLow1, bit *inclusiveHgh1);
crackers_export str CRKMaxTailIncrementalSideways_int_date(int *res, int *bid, int *pbid, int *low1, int *hgh1, bit *inclusiveLow1, bit *inclusiveHgh1);
crackers_export str CRKdjoinSelectSideways_P_int_date(int *res, int *bid, int *pbid, int *pivot, date *low, date *hgh, bit *inclusiveLow, bit *inclusiveHgh);
/* Project */
crackers_export str CRKProjectCrack_int_date(int *res, int *bid, int *pbid, int *low, int *hgh, bit *inclusiveLow, bit *inclusiveHgh);
crackers_export str CRKProjectH_int_date(int *res, int *bid, int *pbid, int *pivot, int *low, int *hgh, bit *inclusiveLow, bit *inclusiveHgh);
crackers_export str CRKProject_int_date(int *res, int *bid, int *pbid, int *pivot, int *low, int *hgh, bit *inclusiveLow, bit *inclusiveHgh);
crackers_export str CRKdProject_int_date(int *res, int *bid, int *pbid, int *pivot);





/* SelectFunctionsSideways */
crackers_export str CRKcreateFullMap_date_int(int* res, int *bid, int *pbid);
crackers_export str CRKhselectBoundsSideways_date_int(int *vid, int *bid, int *pbid, date *low, date *hgh, bit *inclusiveLow, bit *inclusiveHgh);
crackers_export str CRKtselectBoundsSideways_date_int(int *vid, int *bid, int *pbid, date *low, date *hgh, bit *inclusiveLow, bit *inclusiveHgh);
crackers_export str CRKdselectBoundsSideways_date_int(int *vid, int *bid, int *pbid, date *low, date *hgh, bit *inclusiveLow, bit *inclusiveHgh, int *low2, int *hgh2, bit *inclusiveLow2, bit *inclusiveHgh2);
crackers_export str CRKselectBoundsIncrementalSideways_date_int(int *vid, int *bid, int *pbid, date *low, date *hgh, bit *inclusiveLow, bit *inclusiveHgh, int *low2, int *hgh2, bit *inclusiveLow2, bit *inclusiveHgh2);
crackers_export str CRKselectBoundsTailIncrementalSideways_date_int(int *vid, int *bid, int *pbid, date *low, date *hgh, bit *inclusiveLow, bit *inclusiveHgh);
/* JoinSelectFunctionsSideways */
crackers_export str CRKjoinSelectSideways_P_date_int(int *res, int *bid, int *pbid, int *pivot, date *low1, date *hgh1, bit *inclusiveLow1, bit *inclusiveHgh1, int *low, int *hgh, bit *inclusiveLow, bit *inclusiveHgh);
crackers_export str CRKjoinSelectIncrementalSideways_date_int(int *res, int *bid, int *pbid, int *pivot, date *low1, date *hgh1, bit *inclusiveLow1, bit *inclusiveHgh1, int *low, int *hgh, bit *inclusiveLow, bit *inclusiveHgh);
crackers_export str CRKProjectIncrementalSideways_date_int(int *res, int *bid, int *pbid, int *pivot, date *low1, date *hgh1, bit *inclusiveLow1, bit *inclusiveHgh1);
crackers_export str CRKMaxTailIncrementalSideways_date_int(int *res, int *bid, int *pbid, date *low1, date *hgh1, bit *inclusiveLow1, bit *inclusiveHgh1);
crackers_export str CRKdjoinSelectSideways_P_date_int(int *res, int *bid, int *pbid, int *pivot, int *low, int *hgh, bit *inclusiveLow, bit *inclusiveHgh);
/* Project */
crackers_export str CRKProjectCrack_date_int(int *res, int *bid, int *pbid, date *low, date *hgh, bit *inclusiveLow, bit *inclusiveHgh);
crackers_export str CRKProjectH_date_int(int *res, int *bid, int *pbid, int *pivot, date *low, date *hgh, bit *inclusiveLow, bit *inclusiveHgh);
crackers_export str CRKProject_date_int(int *res, int *bid, int *pbid, int *pivot, date *low, date *hgh, bit *inclusiveLow, bit *inclusiveHgh);
crackers_export str CRKdProject_date_int(int *res, int *bid, int *pbid, int *pivot);




/* SelectFunctionsSideways */
crackers_export str CRKcreateFullMap_date_lng(int* res, int *bid, int *pbid);
crackers_export str CRKhselectBoundsSideways_date_lng(int *vid, int *bid, int *pbid, date *low, date *hgh, bit *inclusiveLow, bit *inclusiveHgh);
crackers_export str CRKtselectBoundsSideways_date_lng(int *vid, int *bid, int *pbid, date *low, date *hgh, bit *inclusiveLow, bit *inclusiveHgh);
crackers_export str CRKdselectBoundsSideways_date_lng(int *vid, int *bid, int *pbid, date *low, date *hgh, bit *inclusiveLow, bit *inclusiveHgh, lng *low2, lng *hgh2, bit *inclusiveLow2, bit *inclusiveHgh2);
crackers_export str CRKselectBoundsIncrementalSideways_date_lng(int *vid, int *bid, int *pbid, date *low, date *hgh, bit *inclusiveLow, bit *inclusiveHgh, lng *low2, lng *hgh2, bit *inclusiveLow2, bit *inclusiveHgh2);
crackers_export str CRKselectBoundsTailIncrementalSideways_date_lng(int *vid, int *bid, int *pbid, date *low, date *hgh, bit *inclusiveLow, bit *inclusiveHgh);
/* JoinSelectFunctionsSideways */
crackers_export str CRKjoinSelectSideways_P_date_lng(int *res, int *bid, int *pbid, int *pivot, date *low1, date *hgh1, bit *inclusiveLow1, bit *inclusiveHgh1, lng *low, lng *hgh, bit *inclusiveLow, bit *inclusiveHgh);
crackers_export str CRKjoinSelectIncrementalSideways_date_lng(int *res, int *bid, int *pbid, int *pivot, date *low1, date *hgh1, bit *inclusiveLow1, bit *inclusiveHgh1, lng *low, lng *hgh, bit *inclusiveLow, bit *inclusiveHgh);
crackers_export str CRKProjectIncrementalSideways_date_lng(int *res, int *bid, int *pbid, int *pivot, date *low1, date *hgh1, bit *inclusiveLow1, bit *inclusiveHgh1);
crackers_export str CRKMaxTailIncrementalSideways_date_lng(int *res, int *bid, int *pbid, date *low1, date *hgh1, bit *inclusiveLow1, bit *inclusiveHgh1);
crackers_export str CRKdjoinSelectSideways_P_date_lng(int *res, int *bid, int *pbid, int *pivot, lng *low, lng *hgh, bit *inclusiveLow, bit *inclusiveHgh);
/* Project */
crackers_export str CRKProjectCrack_date_lng(int *res, int *bid, int *pbid, date *low, date *hgh, bit *inclusiveLow, bit *inclusiveHgh);
crackers_export str CRKProjectH_date_lng(int *res, int *bid, int *pbid, int *pivot, date *low, date *hgh, bit *inclusiveLow, bit *inclusiveHgh);
crackers_export str CRKProject_date_lng(int *res, int *bid, int *pbid, int *pivot, date *low, date *hgh, bit *inclusiveLow, bit *inclusiveHgh);
crackers_export str CRKdProject_date_lng(int *res, int *bid, int *pbid, int *pivot);




/* SelectFunctionsSideways */
crackers_export str CRKcreateFullMap_date_date(int* res, int *bid, int *pbid);
crackers_export str CRKhselectBoundsSideways_date_date(int *vid, int *bid, int *pbid, date *low, date *hgh, bit *inclusiveLow, bit *inclusiveHgh);
crackers_export str CRKtselectBoundsSideways_date_date(int *vid, int *bid, int *pbid, date *low, date *hgh, bit *inclusiveLow, bit *inclusiveHgh);
crackers_export str CRKdselectBoundsSideways_date_date(int *vid, int *bid, int *pbid, date *low, date *hgh, bit *inclusiveLow, bit *inclusiveHgh, date *low2, date *hgh2, bit *inclusiveLow2, bit *inclusiveHgh2);
crackers_export str CRKselectBoundsIncrementalSideways_date_date(int *vid, int *bid, int *pbid, date *low, date *hgh, bit *inclusiveLow, bit *inclusiveHgh, date *low2, date *hgh2, bit *inclusiveLow2, bit *inclusiveHgh2);
crackers_export str CRKselectBoundsTailIncrementalSideways_date_date(int *vid, int *bid, int *pbid, date *low, date *hgh, bit *inclusiveLow, bit *inclusiveHgh);
/* JoinSelectFunctionsSideways */
crackers_export str CRKjoinSelectSideways_P_date_date(int *res, int *bid, int *pbid, int *pivot, date *low1, date *hgh1, bit *inclusiveLow1, bit *inclusiveHgh1, date *low, date *hgh, bit *inclusiveLow, bit *inclusiveHgh);
crackers_export str CRKjoinSelectIncrementalSideways_date_date(int *res, int *bid, int *pbid, int *pivot, date *low1, date *hgh1, bit *inclusiveLow1, bit *inclusiveHgh1, date *low, date *hgh, bit *inclusiveLow, bit *inclusiveHgh);
crackers_export str CRKProjectIncrementalSideways_date_date(int *res, int *bid, int *pbid, int *pivot, date *low1, date *hgh1, bit *inclusiveLow1, bit *inclusiveHgh1);
crackers_export str CRKMaxTailIncrementalSideways_date_date(int *res, int *bid, int *pbid, date *low1, date *hgh1, bit *inclusiveLow1, bit *inclusiveHgh1);
crackers_export str CRKdjoinSelectSideways_P_date_date(int *res, int *bid, int *pbid, int *pivot, date *low, date *hgh, bit *inclusiveLow, bit *inclusiveHgh);
/* Project */
crackers_export str CRKProjectCrack_date_date(int *res, int *bid, int *pbid, date *low, date *hgh, bit *inclusiveLow, bit *inclusiveHgh);
crackers_export str CRKProjectH_date_date(int *res, int *bid, int *pbid, int *pivot, date *low, date *hgh, bit *inclusiveLow, bit *inclusiveHgh);
crackers_export str CRKProject_date_date(int *res, int *bid, int *pbid, int *pivot, date *low, date *hgh, bit *inclusiveLow, bit *inclusiveHgh);
crackers_export str CRKdProject_date_date(int *res, int *bid, int *pbid, int *pivot);







#endif /* _CRACKERS_SIDEWAYS_H */

