
#ifndef __MMATH_H__
#define __MMATH_H__
#include "mal_config.h"
#include "mal.h"
#include "mal_exception.h"
#include <math.h>

extern double sqrt(double x);
extern double cbrt(double x);
extern double sin(double x);
extern double cos(double x);
extern double fabs(double x);

#ifdef WIN32
#ifndef LIBMMATH
#define mmath_export extern __declspec(dllimport)
#else
#define mmath_export extern __declspec(dllexport)
#endif
#else
#define mmath_export extern
#endif


#ifdef _MSC_VER
#include <float.h>
/* Windows spells these differently */
#define isnan(x)	_isnan(x)
#define finite(x)	_finite(x)
/* NOTE: HAVE_FPCLASS assumed... */
#define fpclass(x)	_fpclass(x)
#define FP_NINF		_FPCLASS_NINF
#define FP_PINF		_FPCLASS_PINF
#else /* !_MSC_VER */
#ifdef HAVE_IEEEFP_H
#include <ieeefp.h>
#endif
#endif

#if defined(HAVE_FPCLASSIFY) || defined(fpclassify)

/* C99 interface: fpclassify */
#define MNisinf(x)	(fpclassify(x) == FP_INFINITE)
#define MNisnan(x)	(fpclassify(x) == FP_NAN)
#define MNfinite(x)	(!MNisinf(x) && !MNisnan(x))

#else

#define MNisnan(x)	isnan(x)
#define MNfinite(x)	finite(x)

#ifdef HAVE_ISINF

#define MNisinf(x)	isinf(x)

#else

static int
MNisinf(double x)
{
#ifdef HAVE_FPCLASS
	int cl = fpclass(x);

	return ((cl == FP_NINF) || (cl == FP_PINF));
#else
	return 0;		/* XXX not correct if infinite */
#endif /* HAVE_FPCLASS */
}

#endif /* HAVE_ISINF */
#endif /* HAVE_FPCLASSIFY */

#define acos_unary(x, z)      *z = acos(*x)
#define asin_unary(x, z)      *z = asin(*x)
#define atan_unary(x, z)      *z = atan(*x)
#define atan2_binary(x, y, z) *z = atan2(*x,*y)
#define cos_unary(x, z)	      *z = cos(*x)
#define sin_unary(x, z)	      *z = sin(*x)
#define tan_unary(x, z)	      *z = tan(*x)
#define cot_unary(x, z)	      *z = (1/tan(*x))

#define cosh_unary(x, z)       *z = cosh(*x)
#define sinh_unary(x, z)       *z = sinh(*x)
#define tanh_unary(x, z)       *z = tanh(*x)

#define exp_unary(x, z)	      *z = exp(*x)
#define log_unary(x, z)	      *z = log(*x)
#define log10_unary(x, z)     *z = log10(*x)

#define pow_binary(x, y, z)   *z = pow(*x,*y)
#define sqrt_unary(x, z)      *z = sqrt(*x)

#define ceil_unary(x, z)      *z = ((-1.0<*x)&&(*x<0.0))?0.0:ceil(*x)
#define fabs_unary(x, z)      *z = fabs(*x)
#define floor_unary(x, z)     *z = floor(*x)
#define fmod_binary(x, y, z)  *z = fmod(*x,*y)




  
mmath_export str MATHunary_ACOSdbl(dbl *res , dbl *a );



  
mmath_export str MATHunary_ACOSflt(flt *res , flt *a );







  
mmath_export str MATHunary_ASINdbl(dbl *res , dbl *a );



  
mmath_export str MATHunary_ASINflt(flt *res , flt *a );







  
mmath_export str MATHunary_ATANdbl(dbl *res , dbl *a );



  
mmath_export str MATHunary_ATANflt(flt *res , flt *a );







  
mmath_export str MATHbinary_ATAN2dbl(dbl *res, dbl *a, dbl *b );



  
mmath_export str MATHbinary_ATAN2flt(flt *res, flt *a, flt *b );








  
mmath_export str MATHunary_COSdbl(dbl *res , dbl *a );



  
mmath_export str MATHunary_COSflt(flt *res , flt *a );







  
mmath_export str MATHunary_SINdbl(dbl *res , dbl *a );



  
mmath_export str MATHunary_SINflt(flt *res , flt *a );







  
mmath_export str MATHunary_TANdbl(dbl *res , dbl *a );



  
mmath_export str MATHunary_TANflt(flt *res , flt *a );







  
mmath_export str MATHunary_COTdbl(dbl *res , dbl *a );



  
mmath_export str MATHunary_COTflt(flt *res , flt *a );








  
mmath_export str MATHunary_COSHdbl(dbl *res , dbl *a );



  
mmath_export str MATHunary_COSHflt(flt *res , flt *a );







  
mmath_export str MATHunary_SINHdbl(dbl *res , dbl *a );



  
mmath_export str MATHunary_SINHflt(flt *res , flt *a );







  
mmath_export str MATHunary_TANHdbl(dbl *res , dbl *a );



  
mmath_export str MATHunary_TANHflt(flt *res , flt *a );








  
mmath_export str MATHunary_EXPdbl(dbl *res , dbl *a );



  
mmath_export str MATHunary_EXPflt(flt *res , flt *a );







  
mmath_export str MATHunary_LOGdbl(dbl *res , dbl *a );



  
mmath_export str MATHunary_LOGflt(flt *res , flt *a );







  
mmath_export str MATHunary_LOG10dbl(dbl *res , dbl *a );



  
mmath_export str MATHunary_LOG10flt(flt *res , flt *a );








  
mmath_export str MATHbinary_POWdbl(dbl *res, dbl *a, dbl *b );



  
mmath_export str MATHbinary_POWflt(flt *res, flt *a, flt *b );








  
mmath_export str MATHunary_SQRTdbl(dbl *res , dbl *a );



  
mmath_export str MATHunary_SQRTflt(flt *res , flt *a );








  
mmath_export str MATHunary_CEILdbl(dbl *res , dbl *a );



  
mmath_export str MATHunary_CEILflt(flt *res , flt *a );







mmath_export str MATHunary_FABSdbl(dbl *res , dbl *a );




  
mmath_export str MATHunary_FLOORdbl(dbl *res , dbl *a );



  
mmath_export str MATHunary_FLOORflt(flt *res , flt *a );







  
mmath_export str MATHbinary_FMODdbl(dbl *res, dbl *a, dbl *b );



  
mmath_export str MATHbinary_FMODflt(flt *res, flt *a, flt *b );








mmath_export str MATHbinary_ROUNDdbl(dbl *res, dbl *a, int *b );




mmath_export str MATHbinary_ROUNDflt(flt *res, flt *a, int *b );




mmath_export str MATHunary_ISNAN(bit *res, dbl *a);
mmath_export str MATHunary_ISINF(int *res, dbl *a);
mmath_export str MATHunary_FINITE(bit *res, dbl *a);
mmath_export str MATHrandint(int *res);
mmath_export str MATHsrandint(int *seed);
mmath_export str MATHpi(dbl *pi);
#endif /* __MMATH_H__ */


