
#ifndef _MAL_CLIENT_H_
#define _MAL_CLIENT_H_
#define bitset int

/* #define MAL_CLIENT_DEBUG*/

#include "mal_resolve.h"
#include "mal_profiler.h"
#include "mal.h"

#define MAL_MAXCLIENTS  64
#define CONSOLE     0
#define isAdministrator(X) (X==mal_clients)

#define FREECLIENT  0
#define FINISHING   1   
#define CLAIMED     2
#define AWAITING    4

#define TIMEOUT     (5*60)  /* seconds */
#define PROCESSTIMEOUT  2   /* seconds */

#ifdef HAVE_SYS_RESOURCE_H
# include <sys/resource.h>
#endif

#ifdef HAVE_SYS_TIMES_H
# include <sys/times.h>
#endif



typedef struct CLIENT_INPUT {
	bstream             *fdin;
	int                 yycur;		
	int                 listing;
	char                *prompt;
	struct CLIENT_INPUT *next;    
} ClientInput;

typedef struct CLIENT {
	int idx;        /* entry in mal_clients */
	oid user;       /* user id in the auth administration */


	str     scenario;  /* scenario management references */
	str     oldscenario;
	void    *state[7], *oldstate[7];
	MALfcn  phase[7], oldphase[7];
	sht		stage;		/* keep track of the phase being ran */
	char    itrace;     /* trace execution using interactive mdb */
						/* if set to 'S' it will put the process to sleep */
	short   debugOptimizer,debugScheduler;


	sht		flags;	/* resource tracing flags */
	lng     timer;  /* trace time in usec */
	lng		bigfoot; /* maximum virtual memory use */
	lng		vmfoot; /* virtual memory use */

#define timerFlag	1
#define memoryFlag	2
#define ioFlag	4
#define flowFlag	8
#define bigfootFlag	16


	time_t      login;  
	time_t      lastcmd;	/* set when input is received */
	time_t      delay;	/* not yet used */


	str       srcFile;  /* NULL for stdin, or file name */
	bstream  *fdin;
	int       yycur;    /* the scanners current position */


	bitset  listing;        
	str prompt;         /* acknowledge prompt */
	int promptlength;
	ClientInput *bak;   /* used for recursive script and string execution */

	stream   *fdout;    /* streams from and to user. */


	int blkmode;        /* control block parsing */


	bitset debug;
	void  *mdb;            /* context upon suspend */
	str    history;	       /* where to keep console history */
	short  mode;           /* FREECLIENT..BLOCKED */


	MT_Sema 	s;	    /* sema to (de)activate thread */ 
	Thread      	mythread;
	size_t		mypid;
	str     	errbuf;     /* location of GDK exceptions */
	struct CLIENT   *father;    


	Module      nspace;     /* private scope resolution list */
	Symbol      curprg;     /* focus of parser */
	Symbol      backup;     /* save parsing context */
	MalStkPtr   glb;        /* global variable stack */


	int		actions;
	lng		totaltime;	/* sum of elapsed processing times */
#ifdef HAVE_TIMES
	struct tms	workload;
#endif
} *Client, ClientRec;

mal_export ClientRec mal_clients[MAL_MAXCLIENTS+1];

mal_export Client  MCgetClient   (void);
mal_export Client  MCfindClient(oid user, str scen, bstream *fin, stream *fout);
mal_export Client  MCinitClient(oid user, bstream *fin, stream *fout);
mal_export void    MCinitClientThread(Client c);
mal_export void    MCcloseClient    (Client c);
mal_export Client  MCforkClient     (Client c);
mal_export int	   MCcountClients(void);
mal_export int     MCreadClient  (Client c);
mal_export str	   MCsuspendClient(int id, int timeout);
mal_export str	   MCawakeClient(int id);
mal_export int     MCwait(Client c);
mal_export void    MCcleanupClients(void);
mal_export void    MCexitPending(void);
mal_export void    MCtraceAllClients(int flag);
mal_export void    MCtraceClient(oid which, int flag);

mal_export void    MCpushClientInput(Client c, bstream *new_input, int listing, char *prompt);
mal_export void    MCpopClientInput(Client c);

#endif /* _MAL_CLIENT_H_ */

