
#ifndef _TABLET_IO2_H_
#define _TABLET_IO2_H_

#include <gdk.h>
#include "streams.h"
#include <mal_exception.h>
#include <mal_client.h>
#include <mal_interpreter.h>

#ifdef WIN32
#ifndef LIBTABLET
#define tablet_export extern __declspec(dllimport)
#else
#define tablet_export extern __declspec(dllexport)
#endif
#else
#define tablet_export extern
#endif

typedef ptr *(*frStr) (void *extra, int type, char *s, char *e, char quote);
typedef int (*toStr) (void *extra, char **buf, int *len, int type, ptr a);

typedef struct Column_t {
	char *batname;
	char *name;		/* column title */
	char *sep;
	int seplen;
	char *type;
	int adt;		/* type index */
	BAT *c;			/* set to NULL when scalar is meant */
	BATiter ci;
	BUN p;
	unsigned int tabs;	/* field size in tab positions */
	str lbrk, rbrk;		/* column brackets */
	str nullstr;		/* null representation */
	unsigned int width;	/* actual column width */
	unsigned int maxwidth;	/* permissible width */
	int fieldstart;		/* Fixed character field load positions */
	int fieldwidth;
	int scale, precision;
	toStr tostr;
	frStr frstr;
	void *extra;
	void *data;
	int len;
	int nillen;
	bit ws; 	/* if set we need to skip white space */
	bit quote; 	/* if set use this character for string quotes */
} Column;




typedef struct Table_t {
	int signature;
	char *sep;		/* default separator */
	str ttopbrk, tbotbrk;	/* table brackets */
	str rlbrk, rrbrk;	/* row brackets */
	str properties;		/* of header to display */
	str title, footer;	/* alternatives */
	size_t offset;
	size_t nr;		/* allocated space for table loads */
	size_t pageLimit;
	size_t firstrow, lastrow;	/* last window to print */
	size_t nr_attrs;	/* attributes found sofar */
	size_t max_attrs;
	Column *format;		/* remove later */
	stream *fd;
	BAT *pivot;
	str error;	/* last error */
	unsigned int rowwidth;		/* sum of columns used for mallocs */
	Column columns[1];			/* at least one column, enlarged upon need */
} Tablet;

tablet_export BAT *TABLETload(BAT *names, BAT *seps, BAT *types, char *datafile, size_t nr);
tablet_export BAT *TABLETinput(BAT *names, BAT *seps, BAT *types, bstream *s, stream *out, size_t nr, size_t est);
tablet_export size_t TABLEToutput(BAT *order, BAT *seps, BAT *bats, stream *s);
tablet_export void TABLETdump(BAT *names, BAT *seps, BAT *bats, char *datafile, size_t nr);

/* The low level routines are primarilly used by the SQL front-end.*/
tablet_export int TABLETcreate_bats(Tablet * as, size_t est);
tablet_export size_t TABLETassign_BATs(Tablet * as, BAT *bats);
tablet_export ssize_t TABLETload_file(Tablet * as, bstream *b, stream *out);
tablet_export BAT *TABLETcollect_bats(Tablet * as);
tablet_export void TABLETdestroy_format(Tablet * as);
tablet_export int TABLEToutput_file(Tablet * as, BAT *order, stream *s);

tablet_export ptr *TABLETstrFrStr(Column *c, char *s, char *e);
tablet_export ptr *TABLETadt_frStr(void *extra, int type, char *s, char *e, char quote);
tablet_export int TABLETadt_toStr(void *extra, char **buf, int *len, int type, ptr a);
tablet_export int insert_line(Tablet * as, char *line, ptr key);
tablet_export int output_file_dense(Tablet * as, stream *fd);
tablet_export int has_whitespace(char *sep);

#endif

